# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-26 02:38
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import wagtailmetadata.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailimages', '0013_make_rendition_upload_callable'),
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('search_image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page', models.Model),
        ),
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            bases=(wagtailmetadata.models.MetadataMixin, models.Model),
        ),
    ]



from django.conf import settings
from django.template.loader import render_to_string

from wagtailmetadata.models import MetadataSettings


def get_meta_image_url(request, image):
    """
    Resize an image for metadata tags, and return an absolute URL to it.
    """
    rendition = image.get_rendition(filter='original')
    return request.build_absolute_uri(rendition.url)


def meta_tags(request, model):
    context = {
        'site_name': request.site.site_name,
        'object': model,
        'settings': MetadataSettings.for_site(request.site)
    }

    meta_image = model.get_meta_image()
    if meta_image:
        meta_image = get_meta_image_url(request, meta_image)
    context['meta_image'] = meta_image

    return render_to_string('wagtailmetadata/parts/tags.html', context)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-19 04:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmetadata', '0003_auto_20160909_1000'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sitemetadatapreferences',
            name='site',
        ),
        migrations.RemoveField(
            model_name='sitemetadatapreferences',
            name='site_image',
        ),
        migrations.DeleteModel(
            name='SiteMetadataPreferences',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import uuid

from django.forms.utils import flatatt
from django.template import engines
from django.test import RequestFactory, TestCase
from django.utils.html import format_html
from tests.app.models import TestModel, TestPage
from wagtail.wagtailcore.models import Site
from wagtail.wagtailimages.models import Image
from wagtail.wagtailimages.tests.utils import get_test_image_file

from wagtailmetadata.models import MetadataSettings
from wagtailmetadata.tags import get_meta_image_url


class TemplateCase(object):

    def setUp(self):
        self.site = Site.objects.first()
        self.site.site_name = 'Example site'
        self.site.save()
        self.rnd_uuid = uuid.uuid4()
        self.metadata_settings = MetadataSettings.objects.create(
            site=self.site,
            twitter_handle='test',
            fb_app_id=self.rnd_uuid,
        )

        self.factory = RequestFactory()
        self.request = self.factory.get('/test/')
        self.request.site = self.site

        self.image = Image.objects.create(
            title='Test Image',
            file=get_test_image_file(),
        )
        self.page = self.site.root_page.add_child(instance=TestPage(
            title='Test Page',
            search_image=self.image,
            search_description='Some test content description',
        ))
        self.test_model = TestModel.objects.create()

    def render(self, string, context=None, request_context=True):
        if context is None:
            context = {}

        # Add a request to the template, to simulate a RequestContext
        if request_context:
            context['request'] = self.request

        template = self.engine.from_string(string)
        return template.render(context)

    def meta(self, attrs):
        return format_html('<meta{0}>'.format(flatatt(attrs)))

    def test_twitter_render(self):
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'name': 'twitter:card', 'content': 'summary_large_image',
        }), out)
        self.assertInHTML(self.meta({
            'name': 'twitter:title',
            'content': self.page.get_meta_title() + ' — ' + self.site.site_name,
        }), out)
        self.assertInHTML(self.meta({
            'name': 'twitter:description', 'content': self.page.search_description,
        }), out)
        self.assertInHTML(self.meta({
            'name': 'twitter:image',
            'content': get_meta_image_url(self.request, self.page.search_image),
        }), out)

    def test_twitter_no_image(self):
        self.page.search_image = None
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'name': 'twitter:card', 'content': 'summary',
        }), out)
        self.assertNotIn('twitter:image', out)

    def test_og_render(self):
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'property': 'og:url', 'content': self.page.full_url
        }), out)
        self.assertInHTML(self.meta({
            'property': 'og:title',
            'content': self.page.get_meta_title(),
        }), out)
        self.assertInHTML(self.meta({
            'property': 'og:description', 'content': self.page.search_description,
        }), out)
        self.assertInHTML(self.meta({
            'property': 'og:site_name', 'content': self.site.site_name
        }), out)
        self.assertInHTML(self.meta({
            'property': 'og:image',
            'content': get_meta_image_url(self.request, self.page.search_image),
        }), out)

    def test_og_no_image(self):
        self.page.search_image = None
        out = self.render_meta()
        self.assertNotIn('og:image', out)

    def test_fb_app_id(self):
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'property': 'fb:app_id', 'content': self.rnd_uuid
        }), out)

    def test_twitter_handle(self):
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'name': 'twitter:site', 'content': '@test'
        }), out)

    def test_misc_render(self):
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'itemprop': 'url', 'content': self.page.full_url
        }), out)
        self.assertInHTML(self.meta({
            'itemprop': 'name',
            'content': self.page.get_meta_title() + ' — ' + self.site.site_name,
        }), out)
        self.assertInHTML(self.meta({
            'itemprop': 'description', 'content': self.page.search_description,
        }), out)
        self.assertInHTML(self.meta({
            'itemprop': 'image',
            'content': get_meta_image_url(self.request, self.page.search_image),
        }), out)

    def test_generic_render(self):
        out = self.render_meta()
        self.assertInHTML(self.meta({
            'name': 'description', 'content': self.page.search_description,
        }), out)

    def test_custom_model(self):
        out = self.render_with_model()
        self.assertInHTML(self.meta({
            'itemprop': 'url',
            'content': self.test_model.get_meta_url()
        }), out)
        self.assertInHTML(self.meta({
            'itemprop': 'name',
            'content': self.test_model.get_meta_title() + ' — ' + self.site.site_name,
        }), out)
        self.assertInHTML(self.meta({
            'itemprop': 'description',
            'content': self.test_model.get_meta_description()
        }), out)

    def fill_out_page_meta_fields(self):
        self.page.search_description = 'Hello, world'
        self.page.search_image = Image.objects.create(
            title='Page image', file=get_test_image_file())

    def test_page_twitter_render(self):
        self.fill_out_page_meta_fields()

        out = self.render_meta()

        self.assertInHTML(self.meta({
            'name': 'twitter:description', 'content': self.page.search_description,
        }), out)
        self.assertInHTML(self.meta({
            'name': 'twitter:image',
            'content': get_meta_image_url(self.request, self.page.search_image),
        }), out)

    def test_page_og_render(self):
        self.fill_out_page_meta_fields()

        out = self.render_meta()

        self.assertInHTML(self.meta({
            'property': 'og:description', 'content': self.page.search_description,
        }), out)
        self.assertInHTML(self.meta({
            'property': 'og:image',
            'content': get_meta_image_url(self.request, self.page.search_image),
        }), out)

    def test_page_misc_render(self):
        self.fill_out_page_meta_fields()

        out = self.render_meta()

        self.assertInHTML(self.meta({
            'itemprop': 'description', 'content': self.page.search_description,
        }), out)
        self.assertInHTML(self.meta({
            'itemprop': 'image',
            'content': get_meta_image_url(self.request, self.page.search_image),
        }), out)

    def test_page_generic_render(self):
        self.fill_out_page_meta_fields()

        out = self.render_meta()

        self.assertInHTML(self.meta({
            'name': 'description', 'content': self.page.search_description,
        }), out)


class TestJinja2(TemplateCase, TestCase):
    engine = engines['jinja2']

    def render_meta(self):
        return self.render('{{ meta_tags() }}', context={'page': self.page})

    def render_with_model(self):
        return self.render('{{ meta_tags(custom) }}', context={'custom': self.test_model})


class TestDjangoTemplateEngine(TemplateCase, TestCase):
    engine = engines['django']

    def render_meta(self):
        return self.render('{% load wagtailmetadata_tags %}{% meta_tags %}', context={'self': self.page})

    def render_with_model(self):
        return self.render('{% load wagtailmetadata_tags %}{% meta_tags custom %}', context={'custom': self.test_model})



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-07 11:50
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

from wagtailmetadata.utils import get_image_model_string

IMAGE_MODEL = get_image_model_string()


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmetadata', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sitemetadatapreferences',
            name='site_description',
            field=models.TextField(help_text='\n    This is the default description displayed when your page is shared on\n     social media or is found on search engines'),
        ),
        migrations.AlterField(
            model_name='sitemetadatapreferences',
            name='site_image',
            field=models.ForeignKey(blank=True, help_text='\n        This is the default image that will be shown when your page is\n         shared on social media or is found on search engines ', null=True, on_delete=django.db.models.deletion.SET_NULL, to=IMAGE_MODEL),
        ),
    ]



from django.conf import settings


def get_image_model_string():
    try:
        image_model = settings.WAGTAILIMAGES_IMAGE_MODEL
    except AttributeError:
        image_model = 'wagtailimages.Image'
    return image_model



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-11-14 04:53
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailmetadata', '0004_remove_sitemetadatapreferences'),
    ]

    operations = [
        migrations.CreateModel(
            name='MetadataSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('twitter_handle', models.CharField(blank=True, help_text='Not including the @ symbol', max_length=16, null=True)),
                ('fb_app_id', models.CharField(blank=True, max_length=128, null=True)),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.db import models
from django.utils import six
from wagtail.wagtailcore.models import Page

from wagtailmetadata.models import MetadataMixin, MetadataPageMixin


class TestPageBase(type(Page)):
    """See https://code.djangoproject.com/ticket/27337"""
    pass


class TestPage(six.with_metaclass(TestPageBase, MetadataPageMixin, Page)):
    pass


class TestModel(MetadataMixin, models.Model):
    def get_meta_url(self):
        return 'http://takeflight.com.au'

    def get_meta_title(self):
        return 'Sharing on the big blue'

    def get_meta_description(self):
        return 'Wagtail 101 - A journey through a CMS (Corrective Monkey Surgery)'

    def get_meta_image(self):
        return None



from django.db import models
from django.utils.translation import ugettext_lazy
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.wagtailadmin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.wagtailimages.edit_handlers import ImageChooserPanel

from .utils import get_image_model_string


class MetadataMixin(object):
    """
    An object that can be shared on social media.
    """

    def get_meta_url(self):
        """The full URL to this object, including protocol and domain."""
        raise NotImplementedError()

    def get_meta_title(self):
        raise NotImplementedError()

    def get_meta_description(self):
        raise NotImplementedError()

    def get_meta_image(self):
        """
        Get the image to use for this object.
        Can be None if there is no relevant image.
        """
        return None

    def get_meta_twitter_card_type(self):
        """
        Get the Twitter card type for this object.
        See https://dev.twitter.com/cards/types.
        Defaults to 'summary_large_image' if the object has an image,
        otherwise 'summary'.
        """
        if self.get_meta_image() is not None:
            return 'summary_large_image'
        else:
            return 'summary'


class MetadataPageMixin(MetadataMixin, models.Model):
    """An implementation of MetadataMixin for Wagtail pages."""
    search_image = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        related_name='+',
        on_delete=models.SET_NULL
    )

    promote_panels = [
        MultiFieldPanel([
            FieldPanel('slug'),
            FieldPanel('seo_title'),
            FieldPanel('show_in_menus'),
            FieldPanel('search_description'),
            ImageChooserPanel('search_image'),
        ], ugettext_lazy('Common page configuration')),
    ]

    def get_meta_url(self):
        return self.full_url

    def get_meta_title(self):
        return self.seo_title or self.title

    def get_meta_description(self):
        return self.search_description

    def get_meta_image(self):
        return self.search_image

    class Meta:
        abstract = True


@register_setting
class MetadataSettings(BaseSetting):
    twitter_handle = models.CharField(max_length=16, blank=True, null=True, help_text='Not including the @ symbol')
    fb_app_id = models.CharField(max_length=128, blank=True, null=True)

    panels = [
        FieldPanel('twitter_handle'),
        FieldPanel('fb_app_id')
    ]
